(***************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIALYSTOK 2021                               **
**                                                                        **
**             TCPTester - program demostracyjny filozofii sieci          **
**                      Wersja z dnia : 15.04.2021                        **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
***************************************************************************)


unit TCPTesterUnit ;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, blcksock;


type

  { TTCPTesterForm }

  TTCPTesterForm = class(TForm)
    DestIPLabel                  : TLabel ;
    DestPortLabel                : TLabel ;
    DataLabel                    : TLabel ;
    ConnectButton                : TButton ;
    DisconnectButton             : TButton ;
    ExitButton                   : TButton ;
    ClearButton                  : TButton ;
    SendDataButton               : TButton ;
    DestIPEdit                   : TEdit ;
    DestPortEdit                 : TEdit ;
    DataEdit                     : TEdit ;
    StatusBar                    : TStatusBar ;
    GeneralTimer                 : TTimer ;
    TraceMemo                    : TMemo ;
    procedure ClearButtonClick ( Sender : TObject ) ;
    procedure ConnectButtonClick ( Sender : TObject ) ;
    procedure DisconnectButtonClick ( Sender : TObject ) ;
    procedure ExitButtonClick ( Sender : TObject ) ;
    procedure FormCreate ( Sender : TObject ) ;
    procedure GeneralTimerTimer ( Sender : TObject ) ;
    procedure SendDataButtonClick ( Sender : TObject ) ;
  private
    TCPSocket                : TTCPBlockSocket ;
    IPAddress                : String ;
    { private declarations }
  public
    function AcceptOperation : boolean ;
    { public declarations }
  end;

var
  TCPTesterForm: TTCPTesterForm;

implementation

{$R *.lfm}

const
  MachinePanelIndex              = 0 ;
  SrcPanelIndex                  = 1 ;
  DestPanelIndex                 = 2 ;


function ValidPortNo ( Port : String ) : boolean ;
var
  PortNumer             : integer ;

begin (* ValidPortNo *)
  if not TryStrToInt ( Port , PortNumer ) then
  begin (* 1 *)
    ShowMessage ( 'Niepoprawny numer portu.' ) ;
    ValidPortNo := false ;
    exit ;
  end (* 1 *) ;
  if ( PortNumer <= 0 ) or ( PortNumer > 65535 ) then
  begin (* 1 *)
    ShowMessage ( 'Numer portu spaza zakresu' ) ;
    ValidPortNo := false ;
    exit ;
  end (* 1 *) ;
  ValidPortNo := true ;
end (* ValidPortNo *) ;


function TTCPTesterForm . AcceptOperation : boolean ;
begin (* TTCPTesterForm . AcceptOperation *)
  if MessageDlg ( 'Potwierdzenie' , 'Czy potwierdzasz operację?' , mtConfirmation , [ mbYes , mbNo ] , 0 ) = mrYes then
    AcceptOperation := true
  else
    AcceptOperation := false ;
end (* TTCPTesterForm . AcceptOperation *) ;


procedure TTCPTesterForm.FormCreate ( Sender : TObject ) ;
var
  MachineName         : String ;

begin (* TTCPTesterForm.FormCreate *)
  TCPSocket := TTCPBlockSocket . Create ;
  TCPSocket . Family := SF_IP4 ;
  MachineName := TCPSocket . LocalName ;
  IPAddress := TCPSocket . ResolveName ( MachineName ) ;
  TCPSocket . Destroy ;
  StatusBar . Panels . Items [ MachinePanelIndex ] . Text := 'Komputer: ' + MachineName + '/' + IPAddress ;
  DisconnectButton . Enabled := false ;
  SendDataButton . Enabled := false ;
  GeneralTimer . Enabled := false ;
end (* TTCPTesterForm.FormCreate *) ;


procedure TTCPTesterForm.ExitButtonClick ( Sender : TObject ) ;
begin (* TTCPTesterForm.ExitButtonClick *)
  if AcceptOperation then
    Application . Terminate ;
end (* TTCPTesterForm.ExitButtonClick *) ;


procedure TTCPTesterForm.SendDataButtonClick ( Sender : TObject ) ;
var
  Message               : string ;

begin (* TTCPTesterForm.SendDataButtonClick *)
  Message := DataEdit . Text ;
  TCPSocket . SendString ( DataEdit . Text ) ;
  TraceMemo . Lines . Add ( '<' + Message ) ;
end (* TTCPTesterForm.SendDataButtonClick *) ;


procedure TTCPTesterForm.ConnectButtonClick ( Sender : TObject ) ;
var
  DestinationAddress    : string ;
  DestinationPort       : string ;
  LocalPortNo           : integer ;

  function ValidAddress ( Address : String ) : boolean ;
  var
    Index               : cardinal ;
    StrSize             : cardinal ;

    function ExtractNumber : boolean ;
    var
      SubStr            : string ;
      Numb              : integer ;
      Ch                : char ;
      LoopExit          : boolean ;

    begin (* ExtractNumber *)
      SubStr := '' ;
      LoopExit := false ;
      repeat
        if Index > StrSize then
        begin (* 1 *)
          LoopExit := true ;
        end (* 1 *)
        else
        begin (* 1 *)
          Ch := Address [ Index ] ;
          Index := Index + 1 ;
          if Ch = '.' then
            LoopExit := true
          else
            SubStr := SubStr + Ch ;
        end (* 1 *)
      until LoopExit ;
      if Length ( SubStr ) <= 0 then
      begin (* 1 *)
        ExtractNumber := false ;
        exit ;
      end (* 1 *) ;
      if not TryStrToInt ( SubStr , Numb ) then
      begin (* 1 *)
        ExtractNumber := false ;
        exit ;
      end (* 1 *) ;
      if Numb > 255 then
        ExtractNumber := false
      else
        ExtractNumber := true ;
    end (* ExtractNumber *) ;

  begin (* ValidAddress *)
    StrSize := Length ( Address ) ;
    Index := 1 ;
    if not ExtractNumber then
    begin (* 1 *)
      ValidAddress := false ;
      exit ;
    end (* 1 *) ;
    if not ExtractNumber then
    begin (* 1 *)
      ValidAddress := false ;
      exit ;
    end (* 1 *) ;
    if not ExtractNumber then
    begin (* 1 *)
      ValidAddress := false ;
      exit ;
    end (* 1 *) ;
    if not ExtractNumber then
    begin (* 1 *)
      ValidAddress := false ;
      exit ;
    end (* 1 *) ;
    ValidAddress := true ;
  end (* ValidAddress *) ;

begin (* TTCPTesterForm.ConnectButtonClick *)
  DestinationAddress := Trim ( DestIPEdit . Text ) ;
  if Length ( DestinationAddress ) <= 0 then
  begin (* 1 *)
    DestinationAddress := '127.0.0.1' ;
    DestIPEdit . Text := DestinationAddress ;
  end (* 1 *) ;
  if not ValidAddress ( DestinationAddress ) then
  begin (* 1 *)
    ShowMessage ( 'Niepoprawny adres IP.' ) ;
    exit ;
  end (* 1 *) ;
  DestinationPort := Trim ( DestPortEdit . Text ) ;
  if not ValidPortNo ( DestinationPort ) then
    exit ;
  TCPSocket := TTCPBlockSocket . Create ;
  TCPSocket . Family := SF_IP4 ;
  TCPSocket . connect ( DestinationAddress , DestinationPort ) ;
  LocalPortNo := TCPSocket . GetLocalSinPort ;
  if LocalPortNo = 0 then
  begin (* 1 *)
    ShowMessage ( 'Brak jest połączenia ze wskaznym serwerem' ) ;
    TCPSocket . Destroy ;
    exit ;
  end (* 1 *) ;
  TraceMemo . Clear ;
  StatusBar . Panels . Items [ SrcPanelIndex ] . Text := 'Ja: ' + IPAddress + '/' + IntToStr ( LocalPortNo ) ;
  StatusBar . Panels . Items [ DestPanelIndex ] . Text := 'On:' + DestinationAddress + '/' + DestinationPort ;
  GeneralTimer . Enabled := true ;
  DisconnectButton . Enabled := true ;
  SendDataButton . Enabled := true ;
  ConnectButton . Enabled := false ;
  ExitButton . Enabled := false ;
end (* TTCPTesterForm.ConnectButtonClick *) ;


procedure TTCPTesterForm.DisconnectButtonClick ( Sender : TObject ) ;
begin (* TTCPTesterForm.DisconnectButtonClick *)
  if AcceptOperation then
  begin (* 1 *)
    TCPSocket . Destroy ;
    StatusBar . Panels . Items [ SrcPanelIndex ] . Text := '' ;
    StatusBar . Panels . Items [ DestPanelIndex ] . Text := '' ;
    GeneralTimer . Enabled := false ;
    DisconnectButton . Enabled := false ;
    SendDataButton . Enabled := false ;
    ConnectButton . Enabled := true ;
    ExitButton . Enabled := true ;
  end (* 1 *) ;
end (* TTCPTesterForm.DisconnectButtonClick *) ;


procedure TTCPTesterForm.ClearButtonClick ( Sender : TObject ) ;
begin (* TTCPTesterForm.ClearButtonClick *)
  if AcceptOperation then
    TraceMemo . Clear ;
end (* TTCPTesterForm.ClearButtonClick *) ;


procedure TTCPTesterForm.GeneralTimerTimer ( Sender : TObject ) ;
  var
    Data                : String ;

begin (* TTCPTesterForm.GeneralTimerTimer *)
  GeneralTimer . Enabled := false ;
  Data := TCPSocket . RecvPacket ( 0 ) ;
  if length ( Data ) <> 0 then
    TraceMemo . Lines . Add ( '>' + Data ) ;
  GeneralTimer . Enabled := true ;
end (* TTCPTesterForm.GeneralTimerTimer *) ;



end.

